#include "Agent.h"
#include <tinyxml.h>

Agent::Agent()
{
	thoughts.resize(4);
	opponent = NULL;
	br.SetAgent(this);
	perception.SetAgent(this);
	state.SetAgent(this);
	actioninterruptible = true;
}

Agent::~Agent()
{
}

bool 
Agent::LoadActor(const char * filename)
{
	return actor.LoadXML(filename);
}

bool
Agent::LoadProfile(const char * filename)
{
	TiXmlDocument doc;
	if (!doc.LoadFile(filename))
	{
		printf("Error loading profile file: %s\n",doc.ErrorDesc());
		return false;
	}

	TiXmlNode* node = 0;
	TiXmlHandle docHandle( &doc );

	TiXmlElement* profEl = docHandle.FirstChild("profile").Element();
	TiXmlElement* amEl = docHandle.FirstChild("profile").
	                        FirstChild("amatuer").Element();
	TiXmlElement* skEl = docHandle.FirstChild("profile").
	                        FirstChild("skilled").Element();
	TiXmlElement* grEl = docHandle.FirstChild("profile").
	                        FirstChild("defensive").Element();
	TiXmlElement* puEl = docHandle.FirstChild("profile").
	                        FirstChild("offensive").Element();
	TiXmlElement* strEl = docHandle.FirstChild("profile").
	                        FirstChild("strength").Element();

	if (!profEl)
	{
		printf("Bad profile XML format - no profile node\n");
		return false;
	}

	double value;

	if (amEl)
	{
		if (amEl->QueryDoubleAttribute("value", &value) == TIXML_SUCCESS)
			actor.SetAttribute("amateur", (float) value);
	}

	if (skEl)
	{
		if (skEl->QueryDoubleAttribute("value", &value) == TIXML_SUCCESS)
			actor.SetAttribute("skilled", (float) value);
	}

	if (grEl)
	{
		if (grEl->QueryDoubleAttribute("value", &value) == TIXML_SUCCESS)
			actor.SetAttribute("defensive", (float) value);
	}

	if (puEl)
	{
		if (puEl->QueryDoubleAttribute("value", &value) == TIXML_SUCCESS)
			actor.SetAttribute("offensive", (float) value);
	}

	if (strEl)
	{
		if (strEl->QueryDoubleAttribute("value", &value) == TIXML_SUCCESS)
			state.SetStrength((float) value);
	}

	return true;
}

bool
Agent::PlayAction(const char * actionname, bool override)
{
	if (!actor.IsIdle() && timer.Interval(TimeStamp()) < 0.2f)
		return false;	// Slow boxer's reaction time

	if (actor.IsIdle() || override ||
	    (actioninterruptible && currentaction != actionname))
	{
		actor.PlayAction(actionname);
		currentaction = actionname;
		timer.Reset();

		if (currentaction == "turn_left" || currentaction == "turn_right" ||
		    currentaction == "idle")
			actioninterruptible = true;
		else
			actioninterruptible = false;

		return true;
	}
	return false;
}

void
Agent::GetPosition(Vector3 & pos)
{
	actor.GetPos(pos);
}

void 
Agent::SetPosition(const Vector3 & pos)
{
	actor.SetPos(pos);
}

void 
Agent::SetHeading(float h)
{
	actor.SetHeading(h);
}

void 
Agent::AddInputEvent(InputEvent & event)
{
	events.push_back(event);
}

void 
Agent::UpdateAnim()
{
	actor.Update();
}

void 
Agent::Render()
{
	actor.Render();
}

void 
Agent::SetOpponent(Agent * op)
{
	opponent = op;
}

void
Agent::DebugRender()
{
/*	Matrix4x4 myhead;
	Vector3 my_z(0,0,1);

	actor.GetJointXform("head", myhead);
	my_z.Mult(myhead);

    glPushMatrix();
	glDisable(GL_LIGHTING);

	glColor3f(debugcolorr, debugcolorg, debugcolorb);
	glPointSize(2.0);

	glBegin(GL_POINTS);
	glVertex3f(my_z[0], my_z[1], my_z[2]);
	glEnd();

	glEnable(GL_LIGHTING);
    glPopMatrix(); */
}

void
Agent::KeepInRing(float bounds)
{
	Vector3 pelvis;
	float x, z;

	bounds -= 0.25f;

	GetPosition(pelvis);
	x = pelvis[0]; z = pelvis[2];

	if (x < -bounds)
		x = -bounds;
	else if (x > bounds)
		x = bounds;
	if (z < -bounds)
		z = -bounds;
	else if (z > bounds)
		z = bounds;

	if (x != pelvis[0] || z != pelvis[2])
	{
		pelvis.Set(x, 0, z);
		SetPosition(pelvis);
	}
}

void
Agent::Alert(const char * message)
{
	thoughts[0] = message;
}

void
Agent::HandleCollision(std::string myjoint, std::string opjoint)
{
	if (currentaction == "jab" || currentaction == "cross" ||
		currentaction == "hook" || currentaction == "upper_cut")
	{
		if (opjoint != "lhand" && opjoint != "rhand" &&
		    opjoint != "lradius" && opjoint != "rradius")
		{
			ScoreAction(HIT);

			Alert("Attack connected!");
		}
		else
		{
			ScoreAction(OP_BLOCK);

			Alert("Attack blocked!");
		}

		PlayAction("recoil", true);
	}

	if (opponent->GetAction() == "jab" || opponent->GetAction() == "cross" ||
	   opponent->GetAction() == "hook" || opponent->GetAction() == "upper_cut")
	{
		if (myjoint != "lhand" && myjoint != "rhand" &&
		    myjoint != "lradius" && myjoint != "rradius")
		{
			ScoreAction(OP_HIT);

			Alert("Opponent attack connected!");

			if (opponent->GetActor()->GetAttribute("big") > 0.5)
			{
				actor.SetAttribute("small", 0.0f);
				actor.SetAttribute("big", 1.0f);
			}
			else
			{
				actor.SetAttribute("small", 1.0f);
				actor.SetAttribute("big", 0.0f);
			}

			PlayAction("take_hit", true);
		}
		else
		{
			ScoreAction(BLOCK);

			Alert("Opponent attack blocked!");

			PlayAction("recoil", true);
		}

		
	}
}

std::string 
Agent::GetThoughts(const unsigned int line)
{
	if (line < thoughts.size())
		return thoughts[line];

	return "";
}

Actor * 
Agent::GetActor()
{
	return &actor;
}

Agent * 
Agent::GetOpponent()
{
	return opponent;
}

Perception *
Agent::GetPerception()
{
	return &perception;
}

std::string
Agent::GetAction()
{
	return currentaction;
}

State *
Agent::GetState()
{
	return &state;
}